package STM32.Hash_Processor with Pure is

	-- CR

	type Data_Type is (
		Data_32_Bit, -- No swapping of data
		Data_16_Bit, -- Each word consist of two half-words which are swapped with each other
		Data_8_Bit,  -- Each word consist of four bytes in reverse order
		Data_1_Bit   -- Each word is reversed bit-wise
	) with Size => 2;
	for Data_Type use (
		Data_32_Bit => 2#00#,
		Data_16_Bit => 2#01#,
		Data_8_Bit  => 2#10#,
		Data_1_Bit  => 2#11#
	);

	type Hash_Mode is (
		Hash,
		HMAC
	) with Size => 1;
	for Hash_Mode use (
		Hash => 0,
		HMAC => 1
	);

	type Algorithm_0 is mod 2;
	type Algorithm_1 is mod 2;

	type Algorithm is (
		SHA_1,
		MD5,
		SHA224,
		SHA256
	) with Size => 2;
	for Algorithm use (
		SHA_1  => 2#00#,
		MD5    => 2#01#,
		SHA224 => 2#10#,
		SHA256 => 2#11#
	);

	function To_Algorithm_0 (Algo : Algorithm) return Algorithm_0 is (
		Algorithm_0 (Unsigned_32 (Algorithm'Pos (Algo)) and 2#01#)
	);
	-- ALGO0 field value of a given algorithm

	function To_Algorithm_1 (Algo : Algorithm) return Algorithm_1 is (
		Algorithm_1 (Shift_Right (Unsigned_32 (Algorithm'Pos (Algo)), 1))
	);
	-- ALGO1 field value of a given algorithm

	function To_Algorithm (Algo_1 : Algorithm_1; Algo_0 : Algorithm_0) return Algorithm is (
		Algorithm'Val (Shift_Left (Unsigned_32 (Algo_1), 1) or Unsigned_32 (Algo_0))
	);
	-- Composes ALGO0 and ALGO1 to a single value of algorithm

	type Control_Register is record
		Unused_0    : Integer range 0 .. 3  := 0;
		INIT        : Boolean               := False; -- Initialize message digest
		DMAE        : Boolean               := False; -- DMA enable
		DATATYPE    : Data_Type             := Data_32_Bit; -- Data type selection
		MODE        : Hash_Mode             := Hash; -- Mode selection
		ALGO0       : Algorithm_0           := 0; -- Algorithm selection bit 0
		NBW         : Integer range 0 .. 15 := 0; -- Number of words already
		DINNE       : Boolean               := False; -- DIN not empty
		MDMAT       : Boolean               := False; -- Multiple DMA Transfers
		Unused_14   : Integer range 0 .. 3  := 0;
		LKEY        : Boolean               := False; -- Long key selection
		Unused_17   : Integer range 0 .. 1  := 0;
		ALGO1       : Algorithm_1           := 0; -- Algorithm selection bit 1
		Unused_19   : Integer range 0 .. 2**13 - 1 := 0;
	end record with Size => 32;
	for Control_Register use record
		Unused_0    at 0 range  0 ..  1;
		INIT        at 0 range  2 ..  2;
		DMAE        at 0 range  3 ..  3;
		DATATYPE    at 0 range  4 ..  5;
		MODE        at 0 range  6 ..  6;
		ALGO0       at 0 range  7 ..  7;
		NBW         at 0 range  8 .. 11;
		DINNE       at 0 range 12 .. 12;
		MDMAT       at 0 range 13 .. 13;
		Unused_14   at 0 range 14 .. 15;
		LKEY        at 0 range 16 .. 16;
		Unused_17   at 0 range 17 .. 17;
		ALGO1       at 0 range 18 .. 18;
		Unused_19   at 0 range 19 .. 31;
	end record;

	-- STR

	type Start_Register is record
		NBLW        : Integer range 0 .. 2**5 - 1  := 0; -- Number of valid bits in the la
		Unused_5    : Integer range 0 .. 7         := 0;
		DCAL        : Boolean                      := False; -- Digest calculation
		Unused_9    : Integer range 0 .. 2**23 - 1 := 0;
	end record with Size => 32;
	for Start_Register use record
		NBLW        at 0 range  0 ..  4;
		Unused_5    at 0 range  5 ..  7;
		DCAL        at 0 range  8 ..  8;
		Unused_9    at 0 range  9 .. 31;
	end record;

	-- HRx

	type Unsigned_32_Array is array (Integer range <>) of Unsigned_32;

	-- IMR

	type Interrupt_Enable_Register is record
		DINIE       : Boolean := False; -- Data input interrupt enable
		DCIE        : Boolean := False; -- Digest calculation completion interrupt enable
		Unused      : Integer range 0 .. 2**30 - 1 := 0;
	end record with Size => 32;
	for Interrupt_Enable_Register use record
		DINIE       at 0 range  0 ..  0;
		DCIE        at 0 range  1 ..  1;
		Unused      at 0 range  2 .. 31;
	end record;

	-- SR

	type Status_Register is record
		DINIS       : Boolean := True; -- Data input interrupt status
		DCIS        : Boolean := True; -- Digest calculation completion interrupt status
		DMAS        : Boolean := False; -- DMA Status
		BUSY        : Boolean := False; -- Busy bit
		Unused_4    : Integer range 0 .. 2**28 - 1 := 0;
	end record with Size => 32;
	for Status_Register use record
		DINIS       at 0 range  0 ..  0;
		DCIS        at 0 range  1 ..  1;
		DMAS        at 0 range  2 ..  2;
		BUSY        at 0 range  3 ..  3;
		Unused_4    at 0 range  4 .. 31;
	end record;

	-- CSR0

	type CSR0_Register is record
		CSR0        : Unsigned_32; -- CSR0

	end record with Size => 32;
	for CSR0_Register use record
		CSR0        at 0 range  0 .. 31;

	end record;

	-- CSR1

	type CSR1_Register is record
		CSR1        : Unsigned_32; -- CSR1

	end record with Size => 32;
	for CSR1_Register use record
		CSR1        at 0 range  0 .. 31;

	end record;

	-- CSR2

	type CSR2_Register is record
		CSR2        : Unsigned_32; -- CSR2

	end record with Size => 32;
	for CSR2_Register use record
		CSR2        at 0 range  0 .. 31;

	end record;

	-- CSR3

	type CSR3_Register is record
		CSR3        : Unsigned_32; -- CSR3

	end record with Size => 32;
	for CSR3_Register use record
		CSR3        at 0 range  0 .. 31;

	end record;

	-- CSR4

	type CSR4_Register is record
		CSR4        : Unsigned_32; -- CSR4

	end record with Size => 32;
	for CSR4_Register use record
		CSR4        at 0 range  0 .. 31;

	end record;

	-- CSR5

	type CSR5_Register is record
		CSR5        : Unsigned_32; -- CSR5

	end record with Size => 32;
	for CSR5_Register use record
		CSR5        at 0 range  0 .. 31;

	end record;

	-- CSR6

	type CSR6_Register is record
		CSR6        : Unsigned_32; -- CSR6

	end record with Size => 32;
	for CSR6_Register use record
		CSR6        at 0 range  0 .. 31;

	end record;

	-- CSR7

	type CSR7_Register is record
		CSR7        : Unsigned_32; -- CSR7

	end record with Size => 32;
	for CSR7_Register use record
		CSR7        at 0 range  0 .. 31;

	end record;

	-- CSR8

	type CSR8_Register is record
		CSR8        : Unsigned_32; -- CSR8

	end record with Size => 32;
	for CSR8_Register use record
		CSR8        at 0 range  0 .. 31;

	end record;

	-- CSR9

	type CSR9_Register is record
		CSR9        : Unsigned_32; -- CSR9

	end record with Size => 32;
	for CSR9_Register use record
		CSR9        at 0 range  0 .. 31;

	end record;

	-- CSR10

	type CSR10_Register is record
		CSR10       : Unsigned_32; -- CSR10

	end record with Size => 32;
	for CSR10_Register use record
		CSR10       at 0 range  0 .. 31;

	end record;

	-- CSR11

	type CSR11_Register is record
		CSR11       : Unsigned_32; -- CSR11

	end record with Size => 32;
	for CSR11_Register use record
		CSR11       at 0 range  0 .. 31;

	end record;

	-- CSR12

	type CSR12_Register is record
		CSR12       : Unsigned_32; -- CSR12

	end record with Size => 32;
	for CSR12_Register use record
		CSR12       at 0 range  0 .. 31;

	end record;

	-- CSR13

	type CSR13_Register is record
		CSR13       : Unsigned_32; -- CSR13

	end record with Size => 32;
	for CSR13_Register use record
		CSR13       at 0 range  0 .. 31;

	end record;

	-- CSR14

	type CSR14_Register is record
		CSR14       : Unsigned_32; -- CSR14

	end record with Size => 32;
	for CSR14_Register use record
		CSR14       at 0 range  0 .. 31;

	end record;

	-- CSR15

	type CSR15_Register is record
		CSR15       : Unsigned_32; -- CSR15

	end record with Size => 32;
	for CSR15_Register use record
		CSR15       at 0 range  0 .. 31;

	end record;

	-- CSR16

	type CSR16_Register is record
		CSR16       : Unsigned_32; -- CSR16

	end record with Size => 32;
	for CSR16_Register use record
		CSR16       at 0 range  0 .. 31;

	end record;

	-- CSR17

	type CSR17_Register is record
		CSR17       : Unsigned_32; -- CSR17

	end record with Size => 32;
	for CSR17_Register use record
		CSR17       at 0 range  0 .. 31;

	end record;

	-- CSR18

	type CSR18_Register is record
		CSR18       : Unsigned_32; -- CSR18

	end record with Size => 32;
	for CSR18_Register use record
		CSR18       at 0 range  0 .. 31;

	end record;

	-- CSR19

	type CSR19_Register is record
		CSR19       : Unsigned_32; -- CSR19

	end record with Size => 32;
	for CSR19_Register use record
		CSR19       at 0 range  0 .. 31;

	end record;

	-- CSR20

	type CSR20_Register is record
		CSR20       : Unsigned_32; -- CSR20

	end record with Size => 32;
	for CSR20_Register use record
		CSR20       at 0 range  0 .. 31;

	end record;

	-- CSR21

	type CSR21_Register is record
		CSR21       : Unsigned_32; -- CSR21

	end record with Size => 32;
	for CSR21_Register use record
		CSR21       at 0 range  0 .. 31;

	end record;

	-- CSR22

	type CSR22_Register is record
		CSR22       : Unsigned_32; -- CSR22

	end record with Size => 32;
	for CSR22_Register use record
		CSR22       at 0 range  0 .. 31;

	end record;

	-- CSR23

	type CSR23_Register is record
		CSR23       : Unsigned_32; -- CSR23

	end record with Size => 32;
	for CSR23_Register use record
		CSR23       at 0 range  0 .. 31;

	end record;

	-- CSR24

	type CSR24_Register is record
		CSR24       : Unsigned_32; -- CSR24

	end record with Size => 32;
	for CSR24_Register use record
		CSR24       at 0 range  0 .. 31;

	end record;

	-- CSR25

	type CSR25_Register is record
		CSR25       : Unsigned_32; -- CSR25

	end record with Size => 32;
	for CSR25_Register use record
		CSR25       at 0 range  0 .. 31;

	end record;

	-- CSR26

	type CSR26_Register is record
		CSR26       : Unsigned_32; -- CSR26

	end record with Size => 32;
	for CSR26_Register use record
		CSR26       at 0 range  0 .. 31;

	end record;

	-- CSR27

	type CSR27_Register is record
		CSR27       : Unsigned_32; -- CSR27

	end record with Size => 32;
	for CSR27_Register use record
		CSR27       at 0 range  0 .. 31;

	end record;

	-- CSR28

	type CSR28_Register is record
		CSR28       : Unsigned_32; -- CSR28

	end record with Size => 32;
	for CSR28_Register use record
		CSR28       at 0 range  0 .. 31;

	end record;

	-- CSR29

	type CSR29_Register is record
		CSR29       : Unsigned_32; -- CSR29

	end record with Size => 32;
	for CSR29_Register use record
		CSR29       at 0 range  0 .. 31;

	end record;

	-- CSR30

	type CSR30_Register is record
		CSR30       : Unsigned_32; -- CSR30

	end record with Size => 32;
	for CSR30_Register use record
		CSR30       at 0 range  0 .. 31;

	end record;

	-- CSR31

	type CSR31_Register is record
		CSR31       : Unsigned_32; -- CSR31

	end record with Size => 32;
	for CSR31_Register use record
		CSR31       at 0 range  0 .. 31;

	end record;

	-- CSR32

	type CSR32_Register is record
		CSR32       : Unsigned_32; -- CSR32

	end record with Size => 32;
	for CSR32_Register use record
		CSR32       at 0 range  0 .. 31;

	end record;

	-- CSR33

	type CSR33_Register is record
		CSR33       : Unsigned_32; -- CSR33

	end record with Size => 32;
	for CSR33_Register use record
		CSR33       at 0 range  0 .. 31;

	end record;

	-- CSR34

	type CSR34_Register is record
		CSR34       : Unsigned_32; -- CSR34

	end record with Size => 32;
	for CSR34_Register use record
		CSR34       at 0 range  0 .. 31;

	end record;

	-- CSR35

	type CSR35_Register is record
		CSR35       : Unsigned_32; -- CSR35

	end record with Size => 32;
	for CSR35_Register use record
		CSR35       at 0 range  0 .. 31;

	end record;

	-- CSR36

	type CSR36_Register is record
		CSR36       : Unsigned_32; -- CSR36

	end record with Size => 32;
	for CSR36_Register use record
		CSR36       at 0 range  0 .. 31;

	end record;

	-- CSR37

	type CSR37_Register is record
		CSR37       : Unsigned_32; -- CSR37

	end record with Size => 32;
	for CSR37_Register use record
		CSR37       at 0 range  0 .. 31;

	end record;

	-- CSR38

	type CSR38_Register is record
		CSR38       : Unsigned_32; -- CSR38

	end record with Size => 32;
	for CSR38_Register use record
		CSR38       at 0 range  0 .. 31;

	end record;

	-- CSR39

	type CSR39_Register is record
		CSR39       : Unsigned_32; -- CSR39

	end record with Size => 32;
	for CSR39_Register use record
		CSR39       at 0 range  0 .. 31;

	end record;

	-- CSR40

	type CSR40_Register is record
		CSR40       : Unsigned_32; -- CSR40

	end record with Size => 32;
	for CSR40_Register use record
		CSR40       at 0 range  0 .. 31;

	end record;

	-- CSR41

	type CSR41_Register is record
		CSR41       : Unsigned_32; -- CSR41

	end record with Size => 32;
	for CSR41_Register use record
		CSR41       at 0 range  0 .. 31;

	end record;

	-- CSR42

	type CSR42_Register is record
		CSR42       : Unsigned_32; -- CSR42

	end record with Size => 32;
	for CSR42_Register use record
		CSR42       at 0 range  0 .. 31;

	end record;

	-- CSR43

	type CSR43_Register is record
		CSR43       : Unsigned_32; -- CSR43

	end record with Size => 32;
	for CSR43_Register use record
		CSR43       at 0 range  0 .. 31;

	end record;

	-- CSR44

	type CSR44_Register is record
		CSR44       : Unsigned_32; -- CSR44

	end record with Size => 32;
	for CSR44_Register use record
		CSR44       at 0 range  0 .. 31;

	end record;

	-- CSR45

	type CSR45_Register is record
		CSR45       : Unsigned_32; -- CSR45

	end record with Size => 32;
	for CSR45_Register use record
		CSR45       at 0 range  0 .. 31;

	end record;

	-- CSR46

	type CSR46_Register is record
		CSR46       : Unsigned_32; -- CSR46

	end record with Size => 32;
	for CSR46_Register use record
		CSR46       at 0 range  0 .. 31;

	end record;

	-- CSR47

	type CSR47_Register is record
		CSR47       : Unsigned_32; -- CSR47

	end record with Size => 32;
	for CSR47_Register use record
		CSR47       at 0 range  0 .. 31;

	end record;

	-- CSR48

	type CSR48_Register is record
		CSR48       : Unsigned_32; -- CSR48

	end record with Size => 32;
	for CSR48_Register use record
		CSR48       at 0 range  0 .. 31;

	end record;

	-- CSR49

	type CSR49_Register is record
		CSR49       : Unsigned_32; -- CSR49

	end record with Size => 32;
	for CSR49_Register use record
		CSR49       at 0 range  0 .. 31;

	end record;

	-- CSR50

	type CSR50_Register is record
		CSR50       : Unsigned_32; -- CSR50

	end record with Size => 32;
	for CSR50_Register use record
		CSR50       at 0 range  0 .. 31;

	end record;

	-- CSR51

	type CSR51_Register is record
		CSR51       : Unsigned_32; -- CSR51

	end record with Size => 32;
	for CSR51_Register use record
		CSR51       at 0 range  0 .. 31;

	end record;

	-- CSR52

	type CSR52_Register is record
		CSR52       : Unsigned_32; -- CSR52

	end record with Size => 32;
	for CSR52_Register use record
		CSR52       at 0 range  0 .. 31;

	end record;

	-- CSR53

	type CSR53_Register is record
		CSR53       : Unsigned_32; -- CSR53

	end record with Size => 32;
	for CSR53_Register use record
		CSR53       at 0 range  0 .. 31;

	end record;

	type HASH_Registers is record
		CR          : Control_Register;
		DIN         : Unsigned_32;
		STR         : Start_Register;
		HR          : Unsigned_32_Array (0 .. 4);
		IMR         : Interrupt_Enable_Register;
		SR          : Status_Register;
		CSR         : Unsigned_32_Array (0 .. 53);
		HASH_HR     : Unsigned_32_Array (0 .. 7);
	end record;
	for HASH_Registers use record
		CR          at 16#0000# range  0 .. 31;
		DIN         at 16#0004# range  0 .. 31;
		STR         at 16#0008# range  0 .. 31;
		HR          at 16#000C# range  0 .. 32 * 5 - 1;
		IMR         at 16#0020# range  0 .. 31;
		SR          at 16#0024# range  0 .. 31;
		CSR         at 16#00F8# range  0 .. 32 * 54 - 1;
		HASH_HR     at 16#0310# range  0 .. 32 * 8 - 1;
	end record;

end STM32.Hash_Processor;
